#!/usr/bin/env python
import distutils
import inspect
import os
from cached import cached
from getclass import getclass
from isobject import isobject
from isstring import isstring
from public import public


@cached
@public
def isstandardlibrary(obj):
    if obj is None:
        return
    if inspect.ismodule(obj):
        if hasattr(obj, "__file__"):
            path = obj.__file__
            return isstandardlibrary(path)
        else:  # builtin
            return True
    module = None
    if not isobject(obj):
        module = inspect.getmodule(obj)
    if isobject(obj):  # instance
        module = inspect.getmodule(obj)
        cls = getclass(obj)
        if cls:
            return isstandardlibrary(module)
    if module:
        return isstandardlibrary(module)
    if isstring(obj):  # str/unicode
        stdlib = distutils.sysconfig.get_python_lib(standard_lib=True)
        path = os.path.join(stdlib, "%s.py" % obj)
        if os.path.exists(path):
            return True
        try:
            module = __import__(obj)
            if not hasattr(module, "__file__"):
                return True
        except ImportError:
            pass
        if obj.find(".") > 0:  # pkgname.modname
            return isstandardlibrary(os.path.basename(obj).split(".")[0])
        return False
    return False
